# -*- coding: utf-8 -*-

from django.contrib import messages
from django.db.models import Sum, F
from django.db.models.fields import IntegerField
from django.db.models.functions import Cast
from django.shortcuts import render, redirect, get_object_or_404
from django.utils.decorators import method_decorator
from django.utils.safestring import mark_safe
from django.views.generic import FormView
from django_tables2 import SingleTableView

from django.views.generic import CreateView, UpdateView, DeleteView

from ecobox.tables import PointRetraitTable, CommandeTable, FournisseurTable
from lib.access_auth import access_restricted, membre_club, access_club_or_permission
from lib.tools import confirmation, erreur

from portailuser.app_logger import AppLogger
from annuaire.models import requete
from portailuser.models import PortailUser
from datetime import datetime, date, timedelta
from smtplib import SMTPRecipientsRefused
from portail.settings import ECOBOX_MAIL
from django.urls.base import reverse_lazy

from typing import List, Tuple
from dataclasses import dataclass
from decimal import Decimal

from ecobox.models import (
    Ecobox,
    Commande,
    ContenuCommande,
    DateEcobox,
    Blackliste,
    BlacklisteMail,
    PointRetrait,
    Fournisseur,
)
from ecobox.forms import (
    EcoboxForm,
    DateEcoboxForm,
    CommandeAdminForm,
    DateRepetionForm,
    BacklisteMailForm,
    AdresseForm,
    FournisseurForm,
)

_logger = AppLogger("gestion_ecobox")


def blackliste(request):
    if request.user.is_authenticated and Blackliste.objects.filter(personne=request.user):
        mail_bdd = ECOBOX_MAIL[str(request.ecole)]
        msg = mark_safe(
            "Vous avez été interdit d'ecobox. Contactez le BDD pour plus d'informations à l'adresse suivante : <a href=\"mailto:{0}\">{0}</a>".format(
                mail_bdd
            )
        )
        return erreur(request, msg, 403)


def gen_accueil(request):
    aujourdhui = date.today()

    listeecobox = Ecobox.objects.filter(visible=True, ecole=request.ecole)
    if request.user.is_authenticated:
        nb_reserv_non_paye = (
            Commande.objects.filter(user=request.user)
            .filter(paye=False, date__date_livraison__gte=datetime.now() - timedelta(days=7))
            .count()
        )
        peut_gerer = membre_club(request, "bdd") or request.user.check_permission(request, "gerer_ecobox")
    else:
        peut_gerer = False
        # nb_reserv = 0
        nb_reserv_non_paye = 0

    c = {
        "dates": DateEcobox.objects.filter(date_fin_commande__gte=aujourdhui, ecole=request.ecole).order_by(
            "date_fin_commande"
        ),
        # "nb_reserv": nb_reserv,
        "nb_reserv_non_paye": nb_reserv_non_paye,
        "listeecobox": listeecobox,
        "peut_gerer": peut_gerer,
        "ecobox_mail": ECOBOX_MAIL[str(request.ecole)],
    }
    return c


@access_restricted
def commande(request):
    return render(request, "ecobox/accueil.html", gen_accueil(request))


def interface_commande_admin(request):
    if request.user.is_authenticated:
        return redirect("ecobox:commande")

    return render(request, "ecobox/accueil.html", gen_accueil(request))


def vue_commande(request, ident, hashadmin=None):
    commande = get_object_or_404(Commande, pk=ident)

    msg = "Vous n'avez pas la permission d'accéder à cette page"

    if request.user.is_authenticated:
        if request.user != commande.user:
            return erreur(request, msg, "accès non autorisé", 403)
    elif not commande.client_est_inscrit():
        if commande.hashsecure != hashadmin:
            return erreur(request, msg, "accès non autorisé", 403)

    return render(request, "ecobox/valide.html", {"commande": commande})


def annuler_commande(request, ident, hashadmin=None):

    commande_a_sup = get_object_or_404(Commande, pk=ident)

    msg = "Vous n'avez pas la permission d'accéder à cette page"

    if request.user.is_authenticated:
        if request.user != commande_a_sup.user:
            return erreur(request, msg, "accès non autorisé", 403)
    elif not commande_a_sup.client_est_inscrit():
        if commande_a_sup.hashsecure != hashadmin:
            return erreur(request, msg, "accès non autorisé", 403)

    if commande_a_sup.paye:
        msg = "Vous avez payé cette commande, impossible de la supprimer"
        return erreur(request, msg, "Commande payée", 403)

    if commande_a_sup.recupere:
        msg = "Vous avez récupéré cette commande. Impossible de la supprimer. Pensez à la payer"
        return erreur(request, msg, "Commande récupérée", 403)

    if commande_a_sup.date.date_fin_commande < datetime.today():
        mail_bdd = ECOBOX_MAIL[str(request.ecole)]
        msg = mark_safe(
            """La date de fin de réservation est dépassée. Contactez le BDD pour annuler votre commande.
            <a href="mailto:{0}">{0}</a>""".format(
                mail_bdd
            )
        )
        return erreur(request, msg, "date de fin de reservation dépassée", 403)

    # on n'efface pas les preuves !
    bl = blackliste(request)
    if bl:
        return bl

    if request.method == "POST":
        if "conf_ok" in request.POST:
            commande_a_sup.delete()
            if request.user.is_authenticated:
                _logger.log(request.user, "A annulé sa réservation d'écobox.")
        if request.user.is_authenticated:
            return redirect("ecobox:liste_commande")
        else:
            return redirect(interface_commande_admin)

    message = """Êtes-vous sûr de vouloir supprimer cette commande ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]
    return confirmation(request, message, boutons)


@access_restricted
def liste_commande(request):

    commandes = Commande.objects.filter(user=request.user).order_by("-date__date_livraison")

    return render(request, "ecobox/liste_commande.html", {"commandes": commandes})


@access_restricted
def commande_user(request, dt):

    if request.user.get_mobile() == "":
        msg = """Vous devez ajouter un numéro de téléphone mobile sur votre profil pour commander une écobox"""
        return erreur(request, msg, "Impossible de commander", 403)

    bl = blackliste(request)
    if bl:
        return bl

    date_reservation = get_object_or_404(DateEcobox, pk=dt)

    if date_reservation.date_fin_commande < datetime.today():
        msg = """Cette date n'est plus disponible"""
        return erreur(request, msg, "Commande terminée", 403)

    nb_commandes_user = len(Commande.objects.filter(date=date_reservation, user=request.user))

    if request.method == "POST" and "conf_annul" in request.POST:
        messages.error(request, "Commande annulée")
        return redirect("ecobox:commande")
    elif request.method == "GET" and nb_commandes_user > 0:
        msg = """Vous avez déjà effectué une commande pour cette livraison. Souhaitez-vous vraiment commander une nouvelle écobox ?"""
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Confirmer")]
        return confirmation(request, msg, boutons)
    elif "Commander" not in request.POST and ("conf_ok" in request.POST or nb_commandes_user == 0):
        nombres = range(4)
        c = {
            "date_reservation": date_reservation,
            "nombres": nombres,
        }
        return render(request, "ecobox/commande_etudiant.html", c)

    saved = False
    ecobox = date_reservation.ecobox_dispo.filter(disponible=True, visible=True, ecole=request.ecole)
    commande = Commande()
    commande.date = date_reservation
    commande.user = request.user

    point_retrait_pk = request.POST.get("point_retrait")
    if point_retrait_pk:
        point_retrait = PointRetrait.objects.get(pk=point_retrait_pk)
        commande.point_retrait = point_retrait

    for eco in ecobox:
        arg = request.POST.get(f"quantite_{eco.id}")
        if arg and arg.isdigit():
            nb = int(arg)
            if nb > 0:
                if not saved:
                    saved = True
                    commande.save()
                contenu = ContenuCommande()
                contenu.commande = commande
                contenu.ecobox = eco
                contenu.quantite = nb
                contenu.save()
                contenu.synchro_prix()

    if saved:
        _logger.log(request.user, "A effectué une commande d'ecobox.")
        return redirect("ecobox:vue_commande", commande.pk)
    else:
        messages.error(request, "Aucune box commandée")
        return redirect("ecobox:commande")


def commande_admin(request, dt):
    if request.user.is_authenticated:
        return redirect("ecobox:commande_user", dt)

    date_reservation = get_object_or_404(DateEcobox, pk=dt)

    if date_reservation.date_fin_commande < datetime.today():
        msg = """Cette date n'est plus disponible"""
        return erreur(request, msg, "Commande terminée", 403)

    if request.method == "POST":
        form = CommandeAdminForm(request.POST)

        if form.is_valid():
            saved = False
            ecobox = date_reservation.ecobox_dispo.filter(disponible=True, visible=True, ecole=request.ecole)
            commande = form.save(commit=False)
            commande.date = date_reservation
            commande.is_register = False
            commande.gen_hash()

            for eco in ecobox:
                arg = request.POST.get(f"quantite_{eco.id}")
                if arg and arg.isdigit():
                    nb = int(arg)

                    if nb > 0:

                        if not saved:
                            commande.save()
                            saved = True
                        contenu = ContenuCommande()
                        contenu.commande = commande
                        contenu.ecobox = eco
                        contenu.quantite = nb
                        contenu.save()
                        contenu.synchro_prix()
            if saved:
                try:
                    commande.mail_admin(request)
                except SMTPRecipientsRefused:
                    commande.delete()
                    form._errors["mail"] = mark_safe(
                        """<ul class="errorlist"><li>Impossible d'envoyer de mail à l'adresse saisie.</li></ul>"""
                    )
                else:
                    return redirect("ecobox:vue_commande", commande.pk, commande.hashsecure)
            else:
                messages.error(request, "Aucune box commandée")
    else:
        form = CommandeAdminForm()

    nombres = range(4)
    c = {
        "date_reservation": date_reservation,
        "nombres": nombres,
        "form": form,
    }
    return render(request, "ecobox/commande_admin.html", c)


@access_club_or_permission("bdd", "gerer_ecobox")
def valider_paiement(request, cmd):

    commande = get_object_or_404(Commande, pk=cmd)

    if commande.paye:
        if request.method == "POST":
            if "conf_ok" in request.POST:
                commande.paye = False
                commande.save()
                _logger.log(request.user, "a marqué l'ecobox {} comme non payée", commande.pk)
            return redirect("gestion:ecobox:gestion_commandes", commande.date.pk)
        else:
            msg = """Cette commande a déjà été payée. Êtes vous sûr de vouloir la marquer comme non payée"""
            boutons = [("conf_annul", "Annuler"), ("conf_ok", "Valider")]
            return confirmation(request, msg, boutons)
    else:
        commande.paye = True
        commande.save()
        _logger.log(request.user, "a marqué l'ecobox {} comme payée", commande.pk)

    return redirect("gestion:ecobox:gestion_commandes", commande.date.pk)


@access_club_or_permission("bdd", "gerer_ecobox")
def valider_livraison(request, cmd):

    commande = get_object_or_404(Commande, pk=cmd)

    if commande.recupere:
        if request.method == "POST":
            if "conf_ok" in request.POST:
                commande.recupere = False
                commande.save()
                _logger.log(request.user, "a marqué l'ecobox {} comme non livrée", commande.pk)
            return redirect("gestion:ecobox:gestion_commandes", commande.date.pk)
        else:
            msg = """Cette commande a déjà été livrée. Êtes vous sûr de vouloir la marquer comme non livrée ?"""
            boutons = [("conf_annul", "Annuler"), ("conf_ok", "Valider")]
            return confirmation(request, msg, boutons)
    else:
        commande.recupere = True
        commande.save()
        _logger.log(request.user, "a marqué l'ecobox {} comme livrée", commande.pk)

    return redirect("gestion:ecobox:gestion_commandes", commande.date.pk)


@access_club_or_permission("bdd", "gerer_ecobox")
def valider_tout(request, cmd):
    commande = get_object_or_404(Commande, pk=cmd)
    if commande.recupere or commande.paye:
        if request.method == "POST" and "conf_ok" in request.POST:
            commande.recupere = False
            commande.paye = False
            commande.save()
            _logger.log(
                request.user,
                "a marqué l'ecobox {} comme non payée et non livrée",
                commande.pk,
            )
        else:
            msg = """Cette commande a déjà été livrée et payée. Êtes vous sûr de vouloir la marquer comme non payée et non livrée ?"""
            boutons = [("conf_annul", "Annuler"), ("conf_ok", "Valider")]
            return confirmation(request, msg, boutons)
    else:
        commande.recupere = True
        commande.paye = True
        commande.save()
        _logger.log(request.user, "a marqué l'ecobox {} comme payée et livrée", commande.pk)

    return redirect("gestion:ecobox:gestion_commandes", commande.date.pk)


@access_club_or_permission("bdd", "gerer_ecobox")
def annuler_commande_admin(request, com):

    com = get_object_or_404(Commande, pk=com)

    if com.paye or com.recupere:
        msg = """Des validations ont déjà eu lieu sur cette commande. Impossible de la supprimer"""
        return erreur(request, msg, "Suppression impossible", 403)

    if request.method != "POST":
        message = """Êtes-vous sûr de vouloir supprimer cette commande ?"""
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Valider")]
        return confirmation(request, message, boutons)

    dt = com.date

    if "conf_ok" in request.POST:
        com.delete()
        _logger.log(request.user, "a supprimé une ecobox")

    return redirect("gestion:ecobox:gestion_commandes", dt.pk)


@dataclass
class DetailsFournisseur:
    total: Decimal
    commandes: List[Tuple[str, float, float, float, float]]


@method_decorator(access_club_or_permission("bdd", "gerer_ecobox"), name="dispatch")
class GestionCommandesView(SingleTableView):
    model = Commande
    table_class = CommandeTable
    table_pagination = False
    template_name = "ecobox/gestion/liste_commande.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        date_reserv = get_object_or_404(DateEcobox, pk=self.kwargs["date"])
        total_livraison = 0  # prix total à prévoir pour la livraison
        a_commander = {}  # liste des produits à acheter, par fournisseur

        query = (  # dette technique incoming
            ContenuCommande.objects.select_related("ecobox", "commande", "ecobox__fournisseur")
            .filter(quantite__gt=0, commande__date=date_reserv)
            .values_list("ecobox__nom", "prix", "ecobox__fournisseur__nom")
            .annotate(qty=Sum("quantite"), recup=Cast("commande__recupere", IntegerField()))
            .annotate(recups=Sum(F("quantite") * F("recup")))
            .order_by("-ecobox__fournisseur__nom")
        )

        # print(query.query)
        produits = list(query)
        for nom_produit, prix, nom_fournisseur, qty, _, recups in produits:
            if nom_fournisseur not in a_commander:
                a_commander[nom_fournisseur] = DetailsFournisseur(0, [])
            fournisseur = a_commander[nom_fournisseur]

            total_produit = prix * qty
            to_recup = qty - recups
            fournisseur.commandes.append((nom_produit, prix, to_recup, qty, total_produit))

            total_livraison += total_produit
            fournisseur.total += total_produit

        context.update(
            {
                "total_livraison": total_livraison,
                "a_commander": a_commander,
                "date_reservation": date_reserv,
                "now": datetime.now(),
            }
        )
        return context

    def get_queryset(self):
        return Commande.objects.filter(date=self.kwargs["date"])

    def get(self, request, *args, **kwargs):
        if self.kwargs.get("date") is None:
            return redirect("gestion:ecobox:liste_date")
        return super().get(request, *args, **kwargs)


@access_club_or_permission("bdd", "gerer_ecobox")
def mail(request):

    mails = []
    for i in request.POST.getlist("check"):
        addr = get_object_or_404(Commande, pk=i).get_mail()

        if addr:
            mails.append(addr)

    c = {
        "jour": get_object_or_404(Commande, pk=request.POST.getlist("check")[0]).date,
        "lien_mail": "mailto:" + ",".join(mails),
    }

    return render(request, "ecobox/gestion/mail.html", c)


@access_club_or_permission("bdd", "gerer_ecobox")
def sms(request):

    smss = []
    for i in request.POST.getlist("check"):
        numero = get_object_or_404(Commande, pk=i).get_phone()

        if numero:
            smss.append(numero)

    lien_sms = "sms:" + ",".join(smss) + "?body=Vous%20pouvez%20venir%20chercher%20votre%20ecobox"

    c = {
        "jour": get_object_or_404(Commande, pk=request.POST.getlist("check")[0]).date,
        "lien_mail": lien_sms,
    }
    return render(request, "ecobox/gestion/sms.html", c)


@access_club_or_permission("bdd", "gerer_ecobox")
def masse(request):
    if "mail" in request.POST and request.POST.getlist("check"):
        return mail(request)

    if "sms" in request.POST and request.POST.getlist("check"):
        return sms(request)

    return redirect("gestion:ecobox:gestion_commandes")


@access_club_or_permission("bdd", "gerer_ecobox")
def liste_ecobox(request):

    ecobox = Ecobox.objects.all()
    ecobox = ecobox.filter(ecole=request.ecole)

    return render(request, "ecobox/gestion/liste_ecobox.html", {"ecobox": ecobox})


@access_club_or_permission("bdd", "gerer_ecobox")
def edit_ecobox(request, id=None):
    if id:
        ecobox = get_object_or_404(Ecobox, id=id, ecole=request.ecole)
    else:
        ecobox = None

    if request.method == "POST":
        form = EcoboxForm(request.POST, request.FILES, instance=ecobox, ecole=request.ecole)
        if form.is_valid():
            form.save()
            if id:
                _logger.log(request.user, "a edité l'ecobox {}", id)
            else:
                _logger.log(request.user, "a créé l'ecobox {}", form.cleaned_data["nom"])
            return redirect("gestion:ecobox:liste_ecobox")

    else:
        form = EcoboxForm(instance=ecobox, ecole=request.ecole)

    return render(request, "ecobox/gestion/edit_ecobox.html", {"form": form, "ecobox": ecobox})


@access_club_or_permission("bdd", "gerer_ecobox")
def del_ecobox(request, id):
    ecobox = get_object_or_404(Ecobox, id=id, ecole=request.ecole)

    last_month = datetime.today() - timedelta(days=30)
    ref_commandes = ContenuCommande.objects.filter(ecobox=ecobox)
    ref_commandes_recentes = ref_commandes.filter(commande__date_reservation__gte=last_month)

    if ref_commandes and request.method != "POST":
        message = "Cette Écobox est incluse dans {qty} commande{pluriel}".format(
            qty=str(len(ref_commandes)),
            pluriel="s" if len(ref_commandes) > 1 else "",
        )
        if ref_commandes_recentes:
            message += " (dont {qty} date{pluriel} d'il y a moins d'un mois)".format(
                qty=str(len(ref_commandes_recentes)),
                pluriel="nt" if len(ref_commandes_recentes) > 1 else "",
            )
        message += ". Voulez-vous vraiment supprimer cette écobox ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]
        return confirmation(request, message, boutons)

    if request.method != "POST":
        message = """Êtes-vous sûr de vouloir supprimer cette ecobox ?"""
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]
        return confirmation(request, message, boutons)

    if "conf_ok" in request.POST:
        ecobox.delete()
        _logger.log(request.user, "a effacé l'écobox {}", id)

    return redirect("gestion:ecobox:liste_ecobox")


@access_club_or_permission("bdd", "gerer_ecobox")
def liste_date(request):
    dates = DateEcobox.objects.filter(date_livraison__gte=date.today(), ecole=request.ecole).order_by("date_livraison")
    min_date_passe = date.today() - timedelta(days=60)
    dates_passe = DateEcobox.objects.filter(
        date_livraison__lt=date.today(), date_livraison__gt=min_date_passe, ecole=request.ecole
    ).order_by("-date_livraison")
    return render(
        request,
        "ecobox/gestion/liste_date.html",
        {"dates": dates, "dates_passe": dates_passe},
    )


@access_club_or_permission("bdd", "gerer_ecobox")
def edit_date(request, date_edit=None):

    if date_edit:
        date_cmd = get_object_or_404(DateEcobox, pk=date_edit)
    else:
        date_cmd = None

    if request.method == "POST":
        if not date_cmd:
            formrepet = DateRepetionForm(request.POST)
        else:
            formrepet = None
        form = DateEcoboxForm(request.POST, instance=date_cmd, ecole=request.ecole)
        if form.is_valid() and (not formrepet or formrepet.is_valid()):
            dates = form.save(commit=False)
            dates.ecole = str(request.ecole)
            dates.save()
            form.save_m2m()
            if formrepet:
                for i in range(1, formrepet.cleaned_data["repertion"]):
                    new_date = DateEcobox()
                    new_date.date_livraison = dates.date_livraison + timedelta(
                        days=formrepet.cleaned_data["jour_repetition"] * i
                    )
                    new_date.date_fin_commande = dates.date_fin_commande + timedelta(
                        days=formrepet.cleaned_data["jour_repetition"] * i
                    )
                    new_date.ecole = str(request.ecole)
                    new_date.save()

                    for eco in dates.ecobox_dispo.all():
                        new_date.ecobox_dispo.add(eco)

            return redirect("gestion:ecobox:liste_date")
    else:
        form = DateEcoboxForm(instance=date_cmd, ecole=request.ecole)
        if not date_cmd:
            formrepet = DateRepetionForm()
        else:
            formrepet = None

    return render(
        request,
        "ecobox/gestion/edit_date.html",
        {"form": form, "jour": date_cmd, "repet": formrepet},
    )


@access_club_or_permission("bdd", "gerer_ecobox")
def del_date(request, ident):

    ecobox = get_object_or_404(DateEcobox, pk=ident)

    if Commande.objects.filter(date=ecobox):
        msg = "Des commandes existent pour cette date. Impossible de supprimer cette date"
        return erreur(request, msg, "Commandes existantes", 403)

    if request.method != "POST":
        message = """Êtes-vous sûr de vouloir supprimer cette date de commande ?"""
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]
        return confirmation(request, message, boutons)

    if "conf_ok" in request.POST:
        ecobox.delete()

    return redirect("gestion:ecobox:liste_date")


@access_club_or_permission("bdd", "gerer_ecobox")
def liste_black(request):

    texte_rech = request.GET.get("q", "").strip()

    resultats = []

    if texte_rech:
        resultats = requete(texte_rech, request.ecole)
        if not isinstance(resultats, list):
            resultats = [resultats]

    if request.method == "POST":
        form = BacklisteMailForm(request.POST)
        if form.is_valid():
            form.save()
            _logger.log(request.user, "a blacklisté l'adresse {}", form.cleaned_data["mail"])

    else:
        form = BacklisteMailForm()

    c = {
        "badpeople": Blackliste.objects.all(),
        "badmail": BlacklisteMail.objects.all(),
        "form": form,
        "q": texte_rech,
        "resultats": resultats,
    }
    return render(request, "ecobox/gestion/edition_blackliste.html", c)


@access_club_or_permission("bdd", "gerer_ecobox")
def add_black(request, user):

    try:
        utilisateur = PortailUser.objects.get_from_ldap(user)
    except PortailUser.DoesNotExist:
        msg = "L'utilisateur {} n'a pas été trouvé".format(user)
        return erreur(request, msg, 404)

    if not Blackliste.objects.filter(personne=utilisateur):
        black = Blackliste(personne=utilisateur)
        black.save()
        _logger.log(request.user, "a blacklisté {}", user)

    return redirect("gestion:ecobox:liste_black")


@access_club_or_permission("bdd", "gerer_ecobox")
def sup_black(request, user):

    util = get_object_or_404(PortailUser, username=user)
    utilisateur = get_object_or_404(Blackliste, personne=util)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            utilisateur.delete()
            _logger.log(request.user, "a enlevé {} de la blacklist", user)
        return redirect("gestion:ecobox:liste_black")

    message = mark_safe("Êtes-vous sûr de vouloir supprimer {} de la blacklist ?".format(util.get_link()))
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]
    return confirmation(request, message, boutons)


@access_club_or_permission("bdd", "gerer_ecobox")
def sup_mail(request, mail):

    addr = get_object_or_404(BlacklisteMail, pk=mail)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            _logger.log(request.user, "a enlevé {} de la black liste", addr.mail)
            addr.delete()
        return redirect("gestion:ecobox:liste_black")
    message = mark_safe("Êtes-vous sûr de vouloir supprimer {} de la blackliste ?".format(addr.mail))
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]
    return confirmation(request, message, boutons)


@method_decorator(access_club_or_permission("bdd", "gerer_ecobox"), name="dispatch")
class PointRetraitListView(SingleTableView, FormView):
    model = PointRetrait
    table_class = PointRetraitTable
    form_class = AdresseForm
    template_name = "ecobox/gestion/liste_point_retrait.html"

    def form_valid(self, form):
        form.save()
        return redirect("gestion:ecobox:liste_point_retrait")


@access_club_or_permission("bdd", "gerer_ecobox")
def del_point_retrait(request, ident):
    adresse = get_object_or_404(PointRetrait.objects.filter(ecole=request.ecole), pk=ident)
    adresse.delete()
    return redirect("gestion:ecobox:liste_point_retrait")


@method_decorator(access_club_or_permission("bdd", "gerer_ecobox"), name="dispatch")
class FournisseurCreate(CreateView):
    model = Fournisseur
    fields = ["nom"]
    success_url = reverse_lazy("gestion:ecobox:fournisseur_list")
    template_name = "ecobox/gestion/fournisseur_form.html"


@method_decorator(access_club_or_permission("bdd", "gerer_ecobox"), name="dispatch")
class FournisseurUpdate(UpdateView):
    model = Fournisseur
    fields = ["nom"]
    success_url = reverse_lazy("gestion:ecobox:fournisseur_list")
    template_name = "ecobox/gestion/fournisseur_form.html"


@method_decorator(access_club_or_permission("bdd", "gerer_ecobox"), name="dispatch")
class FournisseurDelete(DeleteView):
    model = Fournisseur
    context_object_name = "fournisseur"
    success_url = reverse_lazy("gestion:ecobox:fournisseur_list")

    def get(self, request, *args, **kwargs):
        return self.post(request, *args, **kwargs)


@method_decorator(access_club_or_permission("bdd", "gerer_ecobox"), name="dispatch")
class FournisseurListView(SingleTableView, FormView):
    model = Fournisseur
    table_class = FournisseurTable
    form_class = FournisseurForm
    template_name = "ecobox/gestion/fournisseur.html"

    def form_valid(self, form):
        form.save()
        return redirect("gestion:ecobox:fournisseur_list")
